#include "colors.inc"

// Scene Width - maximum distance between any 2 points
#declare SceneWidth = 1.73205;

// Scene Centre
#declare SceneCentre = <-6.261267546e-018, -6.261267546e-018, -1.08866404e-017>;

// The values below may be changed (original values in brackets):

// Stereo type may be 0 (mono) or 1 (single image stereo)
// or 2 (double image file stereo). If 2 then you must use a
// povray option like +KFF2 which will produce the left and right
// stereo views in separate image files.
#declare StereoType = 0;

// Shadow, 0 - shadows mono no shadows stero,1 - no shadows, 2 - shadows
#declare Shadow = 1;

// Background colour (<0.901961, 0.901961, 0.901961, 0>)
#declare BgColour = <0.901961, 0.901961, 0.901961, 0>; 

// Max Trace Level (10), increase if black areas appear when using transparency
#declare MaxTraceLevel = 10; 

// Aspect Ratio (1.3333) set on command line with Declare=AspectRatio=1.3333
#ifndef(AspectRatio) #declare AspectRatio = 1.33333; #end

// Vertex numbering
#declare TextSize = 0.0866025; // if 0 use value calc'd from geom
#declare TextColour = <0.0392157, 0.0392157, 0.0392157, 0>; //
#declare FontFile = "cyrvetic.ttf"

#ifndef(CameraNumber) #declare CameraNumber = 1; #end

#switch(CameraNumber)
#case (0) // default_camera_0
   // Distance from viewer to LookAt point (2.4248711305964283)
   #declare Distance = 0.9*2.4248711305964283;

   // Rotation Centre
   #declare RotCentre = <-6.261267546e-018, -6.261267546e-018, -1.08866404e-017>;

   // Width of perspective (2) if 0 use default
   #declare PerspFactor = 2;

   // View point, where camera looks (<-6.261267546e-018, -6.261267546e-018, -1.08866404e-017>)
   #declare LookAt = <-6.261267546e-018, -6.261267546e-018, -1.08866404e-017>;

   // Rotation about points centre (<0, -0, 0>)
   #declare Rotation = <0, -0, 0>;

   // Shadow, 0 - shadows mono no shadows stero, 1 - no shadows, 2 - shadows
   #declare Shadow = 1;

   #break

#case (1) // unnamed_camera_1
   // Distance from viewer to LookAt point (2.4248711305964283)
   #declare Distance = 0.9*2.4248711305964283;

   // Rotation Centre
   #declare RotCentre = <-6.261267546e-018, -6.261267546e-018, -1.08866404e-017>;

   // Width of perspective (2) if 0 use default
   #declare PerspFactor = 2;

   // View point, where camera looks (<-6.261267546e-018, -6.261267546e-018, -1.08866404e-017>)
   #declare LookAt = <-6.261267546e-018, -6.261267546e-018, -1.08866404e-017>;

   // Rotation about points centre (<0, -0, 0>)
   #declare Rotation = <0, -0, 0>;

   // Shadow, 0 - shadows mono no shadows stero, 1 - no shadows, 2 - shadows
   #declare Shadow = 1;

   #break

#else
   #debug "CameraNumber out of range"

#end        // switch(CameraNumber)



#macro v_equal(v1,v2) ((v1.x=v2.x)&(v1.y=v2.y)&(v1.z=v2.z)) #end

#macro col_to_tex(col, elem_tex_map, elem_col_map, def_tex)
   #local typ=0;
   #if(col.y>=0) #local typ=1; #end
   #if(!typ & col.y=-1)
      #if(col.x<dimension_size(elem_tex_map,1))
         #ifdef(elem_tex_map[col.x]) #local typ=2; #end
      #end
      #if(!typ & col.x<dimension_size(elem_col_map,1))
         #ifdef(elem_col_map[col.x]) #local typ=3; #end
      #end
   #end
   #switch(typ)
      #case(0) texture{ def_tex } #break;
      #case(1) texture{ pigment{ rgbt col}} #break;
      #case(2) texture{ elem_tex_map[-col.x]} #break;
      #case(3) texture{ pigment{ color elem_col_map[-col.x]}} #break;
   #end
#end


#macro default_disp_vertex(vertex, col)
   sphere{ verts[vertex] vert_sz col_to_tex(col, vert_tex_map, vert_col_map, vert_tex) }
#end


#macro default_disp_edge(edge, col)
   #if(!v_equal(verts[edges[edge][0]], verts[edges[edge][1]]) )
      cylinder{verts[edges[edge][0]] verts[edges[edge][1]] edge_sz
         col_to_tex(col, edge_tex_map, edge_col_map, edge_tex)
      }
   #end
#end


#macro default_disp_face(face_no, idx, col)
   disp_face_triangles(face_no, idx, col)
#end


#macro default_disp_extra()
#end


#macro disp_face_polygon(face_no, idx, col)
   #if (faces[idx]>2)
      polygon{ faces[idx]+1 
      #local i=0;
      #while (i< faces[idx])
         verts[faces[idx+i+1]]
         #local i = i+1;
         #end
      verts[faces[idx+1]]
      col_to_tex(col, face_tex_map, face_col_map, face_tex)
      }
   #end
#end


#macro disp_face_flattened_polygon(face_no, idx, col)
   #local p1 = verts[faces[idx+1]];
   #local i=1;
   #while (i<faces[idx]-1)
      #local norm = vcross(verts[faces[idx+i+1]]-p1,verts[faces[idx+i+2]]-p1);
      #local sin_a = vlength(norm)/(vlength(verts[faces[idx+2]]-p1)*vlength(verts[faces[idx+3]]-p1));
      #if(abs(sin_a)>1e-6)
         #local norm=vnormalize(norm);
         #local i=faces[idx];
      #else
         #local sin_a=0;
      #end
      #local i=i+1;
   #end
   #if(sin_a!=0)
      polygon{ faces[idx]+1 p1
      #local i=1;
      #while (i<faces[idx])
         #local vec = verts[faces[idx+i+1]]-p1;
         p1 + vec - vdot(vec, norm)*norm
         #local i = i+1;
         #end
      p1
      col_to_tex(col, face_tex_map, face_col_map, face_tex)
      }
   #end
#end

#macro disp_face_triangles(face_no, idx, col)
   #local centroid=0;
   #local i=0;
   #while (i< faces[idx])
      #local centroid=centroid+verts[faces[idx+i+1]];
      #local i = i+1;
      #end
   #local centroid=centroid/faces[idx];
   #local i=0;
   union {
   #while (i< faces[idx])
      triangle { centroid verts[faces[idx+i+1]] verts[faces[idx+mod(i+1, faces[idx])+1]] }
      #local i = i+1;
      #end
   col_to_tex(col, face_tex_map, face_col_map, face_tex)
   }
#end


#macro disp_elem_label(pos, txt, col)
   #if(TextSize=0)
      #declare TextSize = SceneWidth/40;
      #end
   text {
      ttf FontFile txt 1 0
      pigment { rgbt col }
      scale <TextSize, TextSize, TextSize/20>
      translate -<TextSize/2, TextSize/2, TextSize/40>
      #declare vec = vnormalize(pos - PtsCentre);
      #declare loc =  pos + (1.2*vert_sz+TextSize)*vec;
      translate SceneCentre + vrotate(loc - SceneCentre, Rotation)
      transform {
         translate -SceneCentre
         rotate Rotation
         translate SceneCentre
         inverse
      }
   }

#end



#if(file_exists("default_pov.inc")) #include "default_pov.inc" #end
#if(file_exists("unnamed.inc")) #include "unnamed.inc" #end

// ###########  non-configurable section ############


#if (StereoType=1 | StereoType=3)
   #declare Distance = Distance*1.4;
#end

#if (PerspFactor=0)
   #if (StereoType=1 | StereoType=3)
      #declare PerspFactor=4;
   #else
      #declare PerspFactor=2;
   #end
#end

// Stereo offset (offset of images or camera from mono position)
#declare StereoOffset = Distance/3.3;

#declare CamLocOff  = <0, 0, Distance*PerspFactor>;

#if (StereoType=2)  // Stereo using separate image file for each view
   #if (clock=0)
      #declare CamLocOff = CamLocOff + < StereoOffset, 0, 0>;
   #else
      #declare CamLocOff = CamLocOff + <-StereoOffset, 0, 0>;
   #end
#end
#if (StereoType=3) // Four views, so move the camera back a bit more
   #declare CamLocOff = CamLocOff*1.2;
#end

#if (StereoType=0|StereoType=2) // Mono, or Stereo using two image files
   #declare ArrSize = 1;
   #declare Offsets = array[ArrSize] {<0, 0, 0>}
#end
#if (StereoType=1)          // Single Image Stereo
   #declare ArrSize = 2;
   #declare Offsets = array[ArrSize] {<+StereoOffset, 0, 0>, <-StereoOffset, 0, 0>}
#end
#if (StereoType=3)          // Tetra View
   #declare ArrSize = 4;
   #declare Offsets = array[ArrSize] {<-StereoOffset, +StereoOffset, 0>, <+StereoOffset, +StereoOffset, 0>, <+StereoOffset, -StereoOffset, 0>, <-StereoOffset, -StereoOffset, 0>}
   #declare OrigRot = Rotation;
   #declare Rots = array [ArrSize] {<0, 0, 0>, <109, 0, 0>, <109, 120, 0>, <109, 240, 0>}
#end

#declare Off=0;
#while(Off<ArrSize)
   #if (StereoType=3)
      #declare Rotation = OrigRot + Rots[Off];
   #end

   object {
      // Objects
      union {
#declare PtsCentre = <-6.261267546e-018, -6.261267546e-018, -1.08866404e-017>;
#declare PtsWidth = 1.73205;
#declare PtsBallRad = 0.146447;

// Display flags
#declare show = 1; // Show object, may be 1 - show, 0 hide

   // Show elements of a type values may be 1 - show, 0 - hide
   #declare verts_show = 1;
   #declare edges_show = 1;
   #declare faces_show = 1;

// Display values
   // Size (or radius) of elements
   #declare vert_sz = 0.00732233; // 0.00732233
   #declare edge_sz = 0.00488155; // 0.00488155
   #declare face_sz = 0; // 0

   // Colour of elements (used to set up default textures
   #declare vert_col = <1, 0.501961, 0, 0>; // <1, 0.501961, 0, 0>
   #declare edge_col = <0.8, 0.6, 0.8, 0>; // <0.8, 0.6, 0.8, 0>
   #declare face_col = <0.8, 0.901961, 0.901961, 0>; // <0.8, 0.901961, 0.901961, 0>

   // Texture of elements
   #declare vert_tex=texture{ pigment{ rgbt vert_col}}
   #declare edge_tex=texture{ pigment{ rgbt edge_col}}
   #declare face_tex=texture{ pigment{ rgbt face_col}}

#declare col_map = array[1]; // Default colourmap
#declare tex_map = array[1]; // Default texmap

#macro disp_vertex(vertex, col)
   default_disp_vertex(vertex, col)
#end

#macro disp_edge(edge, col)
   default_disp_edge(edge, col)
#end

#macro disp_face(face_no, idx, col)
   default_disp_face(face_no, idx, col)
#end

#macro disp_extra()
   default_disp_extra()
#end
// Array of vertex coordinates
#declare num_verts = 24;
#declare verts = array [num_verts] {
   <0.5, 4.908183234841878e-016, 0.7071067811865472>,
   <0.5000000000000002, -0.7071067811865469, -2.555141302941733e-016>,
   <0.4999999999999993, 0.7071067811865478, -4.711380110084309e-017>,
   <-0.4999999999999996, 1.550409106654272e-017, 0.707106781186548>,
   <0.4999999999999996, 1.835554278017959e-016, -0.7071067811865476>,
   <-0.4999999999999994, -0.7071067811865481, -2.304419550554461e-016>,
   <-0.5000000000000003, 0.7071067811865474, 5.655959516250276e-016>,
   <-0.5, -7.616520261710667e-016, -0.7071067811865477>,
   <5.528924285011781e-016, 0.7071067811865475, 0.4999999999999997>,
   <-0.7071067811865471, 1.1998628602952e-016, 0.5>,
   <0.7071067811865477, 3.86324918051091e-017, 0.4999999999999995>,
   <3.38708289326667e-016, 0.7071067811865477, -0.4999999999999999>,
   <-1.396487704583284e-016, -0.7071067811865472, 0.4999999999999999>,
   <-0.707106781186548, -3.161882479613983e-016, -0.4999999999999996>,
   <0.7071067811865476, 1.900955353013342e-016, -0.5000000000000002>,
   <-8.237261311880572e-016, -0.7071067811865479, -0.4999999999999997>,
   <0.7071067811865474, 0.4999999999999999, 4.15778179088137e-016>,
   <-1.480452623013732e-016, 0.5000000000000001, -0.707106781186547>,
   <8.461943521097118e-017, 0.4999999999999994, 0.7071067811865477>,
   <0.7071067811865478, -0.4999999999999997, 1.358913882610042e-016>,
   <-0.7071067811865474, 0.4999999999999997, 6.316813060733442e-017>,
   <-3.621751913672604e-016, -0.4999999999999996, -0.707106781186548>,
   <4.581270836322274e-016, -0.5000000000000002, 0.7071067811865475>,
   <-0.7071067811865478, -0.4999999999999999, -6.86611881775016e-016>
}

// Array of vertex colours
#declare v_cols = array [num_verts]
#declare v_cols[0]=<1, 1, 0, 0>;
#declare v_cols[1]=<1, 1, 0, 0>;
#declare v_cols[2]=<1, 1, 0, 0>;
#declare v_cols[3]=<1, 1, 0, 0>;
#declare v_cols[4]=<1, 1, 0, 0>;
#declare v_cols[5]=<1, 1, 0, 0>;
#declare v_cols[6]=<1, 1, 0, 0>;
#declare v_cols[7]=<1, 1, 0, 0>;
#declare v_cols[8]=<1, 0, 0, 0>;
#declare v_cols[9]=<1, 0, 0, 0>;
#declare v_cols[10]=<1, 0, 0, 0>;
#declare v_cols[11]=<1, 0, 0, 0>;
#declare v_cols[12]=<1, 0, 0, 0>;
#declare v_cols[13]=<1, 0, 0, 0>;
#declare v_cols[14]=<1, 0, 0, 0>;
#declare v_cols[15]=<1, 0, 0, 0>;
#declare v_cols[16]=<0, 0.392157, 0, 0>;
#declare v_cols[17]=<0, 0.392157, 0, 0>;
#declare v_cols[18]=<0, 0.392157, 0, 0>;
#declare v_cols[19]=<0, 0.392157, 0, 0>;
#declare v_cols[20]=<0, 0.392157, 0, 0>;
#declare v_cols[21]=<0, 0.392157, 0, 0>;
#declare v_cols[22]=<0, 0.392157, 0, 0>;
#declare v_cols[23]=<0, 0.392157, 0, 0>;



// Array of edge indexes
#declare num_edges = 54;
#declare edges = array [num_edges][2] {
   {0, 1},
   {0, 2},
   {0, 3},
   {1, 4},
   {1, 5},
   {2, 4},
   {2, 6},
   {3, 5},
   {3, 6},
   {4, 7},
   {5, 7},
   {6, 7},
   {8, 9},
   {8, 10},
   {8, 11},
   {9, 12},
   {9, 13},
   {10, 12},
   {10, 14},
   {11, 13},
   {11, 14},
   {12, 15},
   {13, 15},
   {14, 15},
   {16, 17},
   {16, 18},
   {16, 19},
   {17, 20},
   {17, 21},
   {18, 20},
   {18, 22},
   {19, 21},
   {19, 22},
   {20, 23},
   {21, 23},
   {22, 23},
   {0, 4},
   {0, 6},
   {1, 3},
   {4, 5},
   {2, 7},
   {3, 7},
   {8, 12},
   {8, 14},
   {9, 11},
   {9, 15},
   {10, 15},
   {11, 15},
   {16, 20},
   {16, 22},
   {17, 19},
   {20, 21},
   {18, 23},
   {19, 23}
}

// Array of edge colours
#declare e_cols = array [num_edges]
#declare e_cols[0]=<1, 1, 0, 0>;
#declare e_cols[1]=<1, 1, 0, 0>;
#declare e_cols[2]=<1, 1, 0, 0>;
#declare e_cols[3]=<1, 1, 0, 0>;
#declare e_cols[4]=<1, 1, 0, 0>;
#declare e_cols[5]=<1, 1, 0, 0>;
#declare e_cols[6]=<1, 1, 0, 0>;
#declare e_cols[7]=<1, 1, 0, 0>;
#declare e_cols[8]=<1, 1, 0, 0>;
#declare e_cols[9]=<1, 1, 0, 0>;
#declare e_cols[10]=<1, 1, 0, 0>;
#declare e_cols[11]=<1, 1, 0, 0>;
#declare e_cols[12]=<1, 0, 0, 0>;
#declare e_cols[13]=<1, 0, 0, 0>;
#declare e_cols[14]=<1, 0, 0, 0>;
#declare e_cols[15]=<1, 0, 0, 0>;
#declare e_cols[16]=<1, 0, 0, 0>;
#declare e_cols[17]=<1, 0, 0, 0>;
#declare e_cols[18]=<1, 0, 0, 0>;
#declare e_cols[19]=<1, 0, 0, 0>;
#declare e_cols[20]=<1, 0, 0, 0>;
#declare e_cols[21]=<1, 0, 0, 0>;
#declare e_cols[22]=<1, 0, 0, 0>;
#declare e_cols[23]=<1, 0, 0, 0>;
#declare e_cols[24]=<0, 0.392157, 0, 0>;
#declare e_cols[25]=<0, 0.392157, 0, 0>;
#declare e_cols[26]=<0, 0.392157, 0, 0>;
#declare e_cols[27]=<0, 0.392157, 0, 0>;
#declare e_cols[28]=<0, 0.392157, 0, 0>;
#declare e_cols[29]=<0, 0.392157, 0, 0>;
#declare e_cols[30]=<0, 0.392157, 0, 0>;
#declare e_cols[31]=<0, 0.392157, 0, 0>;
#declare e_cols[32]=<0, 0.392157, 0, 0>;
#declare e_cols[33]=<0, 0.392157, 0, 0>;
#declare e_cols[34]=<0, 0.392157, 0, 0>;
#declare e_cols[35]=<0, 0.392157, 0, 0>;
#declare e_cols[36]=<0, 0, 0, 1>;
#declare e_cols[37]=<0, 0, 0, 1>;
#declare e_cols[38]=<0, 0, 0, 1>;
#declare e_cols[39]=<0, 0, 0, 1>;
#declare e_cols[40]=<0, 0, 0, 1>;
#declare e_cols[41]=<0, 0, 0, 1>;
#declare e_cols[42]=<0, 0, 0, 1>;
#declare e_cols[43]=<0, 0, 0, 1>;
#declare e_cols[44]=<0, 0, 0, 1>;
#declare e_cols[45]=<0, 0, 0, 1>;
#declare e_cols[46]=<0, 0, 0, 1>;
#declare e_cols[47]=<0, 0, 0, 1>;
#declare e_cols[48]=<0, 0, 0, 1>;
#declare e_cols[49]=<0, 0, 0, 1>;
#declare e_cols[50]=<0, 0, 0, 1>;
#declare e_cols[51]=<0, 0, 0, 1>;
#declare e_cols[52]=<0, 0, 0, 1>;
#declare e_cols[53]=<0, 0, 0, 1>;



// Array of face vertex counts and indexes
#declare num_faces = 36;
#declare num_face_items = 144;
#declare faces = array [num_face_items] {
   3, 4, 0, 1, 
   3, 0, 4, 2, 
   3, 6, 0, 2, 
   3, 0, 6, 3, 
   3, 1, 3, 5, 
   3, 3, 1, 0, 
   3, 4, 5, 7, 
   3, 5, 4, 1, 
   3, 2, 7, 6, 
   3, 7, 2, 4, 
   3, 3, 7, 5, 
   3, 7, 3, 6, 
   3, 12, 8, 9, 
   3, 8, 12, 10, 
   3, 14, 8, 10, 
   3, 8, 14, 11, 
   3, 9, 11, 13, 
   3, 11, 9, 8, 
   3, 9, 15, 12, 
   3, 15, 9, 13, 
   3, 10, 15, 14, 
   3, 15, 10, 12, 
   3, 11, 15, 13, 
   3, 15, 11, 14, 
   3, 20, 16, 17, 
   3, 16, 20, 18, 
   3, 22, 16, 18, 
   3, 16, 22, 19, 
   3, 17, 19, 21, 
   3, 19, 17, 16, 
   3, 20, 21, 23, 
   3, 21, 20, 17, 
   3, 18, 23, 22, 
   3, 23, 18, 20, 
   3, 19, 23, 21, 
   3, 23, 19, 22
}

// Array of face colours
#declare f_cols = array [num_faces]
#declare f_cols[0]=<1, 1, 0, 0>;
#declare f_cols[1]=<1, 1, 0, 0>;
#declare f_cols[2]=<1, 1, 0, 0>;
#declare f_cols[3]=<1, 1, 0, 0>;
#declare f_cols[4]=<1, 1, 0, 0>;
#declare f_cols[5]=<1, 1, 0, 0>;
#declare f_cols[6]=<1, 1, 0, 0>;
#declare f_cols[7]=<1, 1, 0, 0>;
#declare f_cols[8]=<1, 1, 0, 0>;
#declare f_cols[9]=<1, 1, 0, 0>;
#declare f_cols[10]=<1, 1, 0, 0>;
#declare f_cols[11]=<1, 1, 0, 0>;
#declare f_cols[12]=<1, 0, 0, 0>;
#declare f_cols[13]=<1, 0, 0, 0>;
#declare f_cols[14]=<1, 0, 0, 0>;
#declare f_cols[15]=<1, 0, 0, 0>;
#declare f_cols[16]=<1, 0, 0, 0>;
#declare f_cols[17]=<1, 0, 0, 0>;
#declare f_cols[18]=<1, 0, 0, 0>;
#declare f_cols[19]=<1, 0, 0, 0>;
#declare f_cols[20]=<1, 0, 0, 0>;
#declare f_cols[21]=<1, 0, 0, 0>;
#declare f_cols[22]=<1, 0, 0, 0>;
#declare f_cols[23]=<1, 0, 0, 0>;
#declare f_cols[24]=<0, 0.392157, 0, 0>;
#declare f_cols[25]=<0, 0.392157, 0, 0>;
#declare f_cols[26]=<0, 0.392157, 0, 0>;
#declare f_cols[27]=<0, 0.392157, 0, 0>;
#declare f_cols[28]=<0, 0.392157, 0, 0>;
#declare f_cols[29]=<0, 0.392157, 0, 0>;
#declare f_cols[30]=<0, 0.392157, 0, 0>;
#declare f_cols[31]=<0, 0.392157, 0, 0>;
#declare f_cols[32]=<0, 0.392157, 0, 0>;
#declare f_cols[33]=<0, 0.392157, 0, 0>;
#declare f_cols[34]=<0, 0.392157, 0, 0>;
#declare f_cols[35]=<0, 0.392157, 0, 0>;



// Colour Maps - redefine these, normally in an include file
   #declare col_map = array[1];
   #declare tex_map = array[1];
   #declare vert_col_map = col_map;
   #declare vert_tex_map = tex_map;
   #declare edge_col_map = col_map;
   #declare edge_tex_map = tex_map;
   #declare face_col_map = col_map;
   #declare face_tex_map = tex_map;

#if(file_exists("default_off_i.inc")) #include "default_off_i.inc" #end
#if(file_exists("uc8_i.inc")) #include "uc8_i.inc" #end

#if (show)
#declare NoColour = <-1, -1, -1, 0>; // Indicates no colour has been set// Display vertex elements
#if(verts_show)
   #declare i=0;
   #while (i<num_verts)
      #declare col = NoColour;
      #ifdef (v_cols[i]) #declare col=v_cols[i]+<0,0,0,0>; #end
         #if (col.x!=0 | col.y!=0 | col.z!=0 | col.t!=1)
            disp_vertex(i, col)
         #end
      #declare i=i+1;
      #end
   #end // (verts_show)

// Display edge elements
#if (edges_show)
   #declare i=0;
   #while (i<num_edges)
      #declare col = NoColour;
      #ifdef (e_cols[i]) #declare col=e_cols[i]+<0,0,0,0>; #end
         #if (col.x!=0 | col.y!=0 | col.z!=0 | col.t!=1)
            disp_edge(i, col)
         #end
      #declare i=i+1;
      #end
   #end // (edges_show)

// Display face elements
#if (faces_show)
   #declare face_no=0;   #declare idx=0;
   #while (face_no<num_faces)
      #declare col = NoColour;
      #ifdef (f_cols[face_no]) #declare col=f_cols[face_no]+<0,0,0,0>; #end
         #if (col.x!=0 | col.y!=0 | col.z!=0 | col.t!=1)
            disp_face(face_no, idx, col)
         #end
      #declare idx = idx + faces[idx] + 1;
      #declare face_no=face_no+1;
      #end
   #end // (faces_show)

// Extra object
disp_extra()

#end // (show)
// Label display flags
#declare show = 1; // Show object, may be 1 - show, 0 hide

   // Show elements of a type values may be 1 - show, 0 - hide
   #declare vert_labs_show = 0;
   #declare edge_labs_show = 0;
   #declare face_labs_show = 0;

   // Label colour for elements (used to set up default textures
   #declare vert_lab_col = <0.501961, 0, 0, 0>; // <0.501961, 0, 0, 0>
   #declare edge_lab_col = <0, 0.501961, 0, 0>; // <0, 0.501961, 0, 0>
   #declare face_lab_col = <0, 0, 0.501961, 0>; // <0, 0, 0.501961, 0>

#if (show)
#declare NoColour = <-1, -1, -1, 0>; // Indicates no colour has been set// Display vertex elements
#if(vert_labs_show)
   #declare i=0;
   #while (i<num_verts)
      disp_elem_label(verts[i], str(i, 0, 0), vert_lab_col) 
      #declare i=i+1;
      #end
   #end // (vert_labs_show)

// Display edge elements
#if (edge_labs_show)
   #declare i=0;
   #while (i<num_edges)
      #declare centroid = (edge[i][0] + edge[i][1])/2.0;
      disp_elem_label(centroid, str(i, 0, 0), vert_lab_col) 
      #declare i=i+1;
      #end
   #end // (edges_show)

// Display face elements
#if (face_labs_show)
   #declare face_no=0;   #declare idx=0;
   #while (face_no<num_faces)
      #local centroid=0;
      #local i=0;
      #while (i< faces[idx])
         #local centroid=centroid+verts[faces[idx+i+1]];
         #local i = i+1;
         #end
      #local centroid=centroid/faces[idx];
      disp_elem_label(centroid, str(i, 0, 0), vert_lab_col) 
      #declare idx = idx + faces[idx] + 1;
      #declare face_no=face_no+1;
      #end
   #end // (face_labs_show)
#end // (show)


         translate - SceneCentre
         rotate Rotation
         translate SceneCentre + Offsets[Off]
      }

      // Don't do slow clipping unless there is chance of overlap
      #if ((StereoType=1|StereoType=3) & ((Distance < 1.2*SceneWidth*1.4) |(vlength(LookAt-SceneCentre))) )
         bounded_by {
            box { SceneWidth*(<1, 1, 1>), -SceneWidth*<1, 1, 1>
               translate SceneWidth*<select(Offsets[Off].x, -1, 0, 1), select(Offsets[Off].y, -1, 0, 1), 0>
               translate Distance*PerspFactor*z
               rotate <-3*atan(Offsets[Off].y/(Distance*PerspFactor)), 3*atan(Offsets[Off].x/(Distance*PerspFactor)),0>
               translate -Distance*PerspFactor*z+LookAt
            }
         }
         clipped_by { bounded_by }
      #end
   } // close of object
   #declare Off = Off+1;
#end


background {color BgColour}
global_settings{max_trace_level MaxTraceLevel}

#ifndef(ExcludeDefCamera) #declare ExcludeDefCamera=0; #end
#if(!ExcludeDefCamera)
  camera { location LookAt + CamLocOff
           look_at LookAt
           direction <0 , 0, PerspFactor>
           right <-1*AspectRatio , 0, 0>
         }
#end


#ifndef(ExcludeDefLights) #declare ExcludeDefLights=0; #end
#if(!ExcludeDefLights)
   light_source {LookAt + <0, 0, SceneWidth*10> color <0.7, 0.7, 0.7> 
      #if (Shadow=1|(!Shadow & StereoType))
         shadowless
      #end
   }

   light_source {LookAt + <0 ,2*SceneWidth, SceneWidth*10> color White 
      #if (Shadow=1|(!Shadow & StereoType))
         shadowless
      #end
   }
#end

